#include <tizen.h>
#include <service_app.h>
#include <account.h>
#include <account-types.h>
#include <account-error.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>
#include <libxml/tree.h>
#include <sync_adapter.h>
#include <bundle.h>
#include <appfw/package_manager.h>
#include "sync-adapter-service-app.h"


#define SYNC_ADAPTER_APP_ID "org.tizen.syncadapterapp.ui"


struct MemoryStruct {
	char *memory;
	size_t size;
};

static int count = 0;
int page_count = 0, sync_count = 0, clear_flag = 0;
char *pageContents;


static void handleStopSync(account_h account, const char *capability)
{
	dlog_print(DLOG_INFO, LOG_TAG, "HandleStopSync called in service app");

	int id;
	account_get_account_id(account, &id);
	dlog_print(DLOG_INFO, LOG_TAG, "details %d, %s ", id, capability);
	dlog_print(DLOG_INFO, LOG_TAG, "HandleStopSync called in client");
}


static bool handleStartSync(account_h account, const char *sync_job_name, const char *sync_capability, bundle *sync_job_user_data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "HandleStartSync called in service app");

	bool is_data_change = sync_capability ? true : false;

	if (sync_job_name)
		dlog_print(DLOG_INFO, LOG_TAG, "Sync Job Name [%s]", sync_job_name);
	else
		dlog_print(DLOG_INFO, LOG_TAG, "Data Change Sync Job Capability [%s]", sync_capability);

	if (account) {
		int id;
		account_get_account_id(account, &id);
		dlog_print(DLOG_INFO, LOG_TAG, "account [%d] ", id);
	} else
		dlog_print(DLOG_INFO, LOG_TAG, "accountless case");

	app_control_h app_control;
	int ret = app_control_create(&app_control);
	ret = app_control_set_app_id(app_control, SYNC_ADAPTER_APP_ID);
	if (ret == APP_CONTROL_ERROR_NONE)
		dlog_print(DLOG_INFO, LOG_TAG, "Launching sync UI app successfully [%s]", SYNC_ADAPTER_APP_ID);
	else
		dlog_print(DLOG_INFO, LOG_TAG, "Launching sync UI app failed [%s]", get_error_message(ret));

	if (is_data_change) {
		ret = app_control_set_operation(app_control, "http://tizen.org/appcontrol/operation/data_change_sync_complete");
	} else {
		bool is_periodic = !strcmp(sync_job_name, "Periodic") ? true : false;
		if (is_periodic)
			ret = app_control_set_operation(app_control, "http://tizen.org/appcontrol/operation/periodic_sync_complete");
		else
			ret = app_control_set_operation(app_control, "http://tizen.org/appcontrol/operation/on_demand_sync_complete");
	}
	dlog_print(DLOG_INFO, LOG_TAG, "sync operation is completed");

	app_control_send_launch_request(app_control, NULL, NULL);

	return true;
}


bool service_app_create(void *data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "service_app_create called");

	sync_adapter_set_callbacks(handleStartSync, handleStopSync);

	return true;
}


void service_app_terminate(void *data)
{
	return;
}


void service_app_control(app_control_h app_control, void *data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "service_app_control called");

	char *app_id = NULL;
	int ret = app_control_get_app_id(app_control, &app_id);

	if (ret != APP_CONTROL_ERROR_NONE) {
		dlog_print(DLOG_INFO, LOG_TAG, "getting app id is failed: [%s]", get_error_message(ret));
		return;
	}

	if (app_id != NULL) {
		dlog_print(DLOG_INFO, LOG_TAG, "caller %s", app_id);
		free(app_id);
	}

	count = 0;

	return;
}


static void
service_app_low_battery(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_BATTERY*/
}


static void
service_app_low_memory(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_MEMORY*/
}

static void
service_app_orient_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_DEVICE_ORIENTATION_CHANGED*/
	return;
}


static void
service_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	return;
}


static void
service_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_REGION_FORMAT_CHANGED*/
}


int main(int argc, char *argv[])
{
	char ad[50] = {0,};
	service_app_lifecycle_callback_s event_callback;
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = service_app_create;
	event_callback.terminate = service_app_terminate;
	event_callback.app_control = service_app_control;

	service_app_add_event_handler(&handlers[APP_EVENT_LOW_BATTERY], APP_EVENT_LOW_BATTERY, service_app_low_battery, &ad);
	service_app_add_event_handler(&handlers[APP_EVENT_LOW_MEMORY], APP_EVENT_LOW_MEMORY, service_app_low_memory, &ad);
	service_app_add_event_handler(&handlers[APP_EVENT_DEVICE_ORIENTATION_CHANGED], APP_EVENT_DEVICE_ORIENTATION_CHANGED, service_app_orient_changed, &ad);
	service_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, service_app_lang_changed, &ad);
	service_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED], APP_EVENT_REGION_FORMAT_CHANGED, service_app_region_changed, &ad);

	return service_app_main(argc, argv, &event_callback, ad);
}
